﻿using System;
using System.ComponentModel;
using System.Threading;
using System.Windows.Forms;
using HeuristicLab.Clients.Hive.CloudManager.Model;

namespace HeuristicLab.Clients.Hive.CloudManager.Views {
  public partial class AddSubscriptionDialog : Form {
    public bool ErrorOccured { get; set; }
    public Subscription Subscription { get; set; }

    private BackgroundWorker worker = new BackgroundWorker();

    public AddSubscriptionDialog() {
      InitializeComponent();

      txtCertThumbprint.Text = "EFFD1808EF2F0658C52997CF00F7A86680F9FA62";
      txtSubscriptionId.Text = "271e67e2-3132-4429-b0e7-ab94d3a95554";
      cbSaveSubscription.Checked = false;
      cbDiscoverServices.Checked = true;

      SetWorkerEvents();
      ErrorOccured = true;
    }

    private void btnOk_Click(object sender, System.EventArgs e) {
      this.progressBar.Visible = true;
      //this.progressBar.Style = ProgressBarStyle.Continuous;
      worker.RunWorkerAsync();
    }

    private void btnCancel_Click(object sender, System.EventArgs e) {
      this.Close();
    }

    private void SetWorkerEvents() {
      worker.WorkerReportsProgress = true;
      worker.DoWork += new DoWorkEventHandler(LoadSubscription);
      worker.ProgressChanged += new ProgressChangedEventHandler(ProgressChanged);
      worker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(WorkerCompleted);
    }

    private void ProgressChanged(object sender, ProgressChangedEventArgs e) {
      progressBar.Value = e.ProgressPercentage;
    }

    private void WorkerCompleted(object sender, RunWorkerCompletedEventArgs e) {
      progressBar.Visible = false;
      if (!ErrorOccured) {
        this.Close();
      } else {
        this.Show();
      }
    }

    private void LoadSubscription(object sender, DoWorkEventArgs e) {
      try {
        worker.ReportProgress(25, "Add Subscription");
        Subscription = CloudManagerClient.Instance.AzureProvider.GetSubscriptionInfo(txtSubscriptionId.Text, txtCertThumbprint.Text);
        Subscription.SaveToConfig = cbSaveSubscription.Checked;
        Subscription.DiscoverServices = cbDiscoverServices.Checked;
        ErrorOccured = false;
      }
      catch (SystemException ex) {
        ErrorOccured = true;
        MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
      try {

        if (!ErrorOccured) {
          worker.ReportProgress(50, "Get Subscription Info");
          //TODO: operation
          ErrorOccured = false;
        }
      }
      catch (SystemException ex) {
        ErrorOccured = true;
        MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        worker.ReportProgress(25, "Rollback: Remove hosted service");
        // Rollback operation
      }

      if (!ErrorOccured) {
        worker.ReportProgress(100, "");
        Thread.Sleep(1000);
      }
    }
  }
}
